//
// Created by Alexander Rupsch on 31/10/14.
// Copyright (c) 2014 - 2015 TomTom Telematics. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "TTLServiceRealTimeDataSubscriptionProtocol.h"


#pragma mark - TTLServiceOBDRealTimeDelegate

/** Representation of Real time data chunk
*
*/
@interface TTLServiceOBDRealTimeDatagram : NSObject <NSCopying, TTLOBDDatagram, TTLAccelerationDatagram>
/** Reference id
*
* This is unique for every LINK-Device. Note: **This does not match the device identifier!**
*
* @see [TTLServiceOBDRealTimeDelegate onObdRealTimeDatagram:deviceId:]
*/
@property(nonatomic, strong) NSNumber * _Nullable refId;

- (NSString * _Nonnull)description;

- (BOOL)isEqual:(id _Nullable)other;

- (BOOL)isEqualToDatagram:(TTLServiceOBDRealTimeDatagram * _Nullable)datagram;

- (NSUInteger)hash;

- (id _Nonnull)copyWithZone:(NSZone * _Nullable)zone;

@end

@interface TTLServiceMutableOBDRealTimeDatagram : TTLServiceOBDRealTimeDatagram
/** Virtual odometer */
@property(readwrite) NSNumber * _Nullable odometer;
/** Fuel system status */
@property(readwrite) NSNumber * _Nullable fuelSystemStatus;
/** Calculated engine load */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable engineLoad;
/** Engine coolant temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable coolantTemperature;
/** Intake manifold pressure */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable manifoldPressure;
/** Engine RPM */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable engineRpm;
/** Vehicle speed */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable vehicleSpeed;
/** Intake air temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable airTemperature;
/** Throttle position */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable throttlePosition;
/** Mass air flow */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable massAirFlow;
/** Fuel rate */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable fuelRate;
/** Fuel level in percent */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable fuelLevel;
/** Battery voltage */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable batteryVoltage;
/** Fuel rail pressure */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable fuelRailPressure;
/** Commanded exhaust gas recirculation */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable commandedExhaustGasRecirculation;
/** Barometric pressure */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable barometricPressure;
/** Catalyst temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable catalystTemperature;
/** Absolute load value */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable absoluteLoad;
/** Ambient air temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable ambientAirTemperature;
/** Engine oil temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable engineOilTemperature;
/** Charge air cooler temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable chargeAirCoolerTemperature;
/** Exhaust gas temperature */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable exhaustGasTemperature;

/** The time when the chunk arrived at the smart phone as unix timestamp. */
@property(nonatomic, strong, readwrite) NSDate * _Nullable receivingTime;

/** Acceleration */
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationAvgX;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationAvgY;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationAvgZ;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMaxX;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMaxY;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMaxZ;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMinX;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMinY;
@property(nonatomic, strong, readwrite) NSNumber * _Nullable accelerationMinZ;

@end

/** Real time data response used with [TTLServiceOBDRealTimeDelegate onObdRealTimeDatagram:deviceId:] which is called when a new
* real time datagram is available.
*
*/
@interface TTLServiceOBDRealTimeDatagramResponse : NSObject
/** A datagram encapsulating the OBD real time information. */
@property TTLServiceOBDRealTimeDatagram * _Nonnull obdRealTimeDatagram;

- (NSString * _Nonnull)description;
@end

@protocol TTLServiceOBDRealTimeDelegate <NSObject>

/** Is called when the LINK device sends a real time datagram to the smart phone.

@param response A response container carrying the OBD real time datagram.
@param deviceIdentifier The device identifier for which the event occurs.
*/
- (void)onObdRealTimeDatagram:(TTLServiceOBDRealTimeDatagramResponse * _Nonnull)response deviceId:(NSString * _Nonnull)deviceIdentifier;
@end

#pragma mark - TTLServiceDrivingProfileRealTimeDelegate


/** A profile of driving behaviour related to a certain time frame.
*
* The profile contains the count of all events that happen in that
* time frame.
*
* The time frame models a tour that ends after the engine is turned off
* "long enough". Currently the time out is about 2h.
*/
@interface TTLServiceDrivingProfileDatagram : NSObject <NSCopying, NSMutableCopying, TTLDrivingProfileDatagram>

- (NSString * _Nonnull)description;

- (id _Nonnull)copyWithZone:(NSZone * _Nullable)zone;

- (BOOL)isEqual:(id _Nullable)other;

- (BOOL)isEqualToDatagram:(TTLServiceDrivingProfileDatagram * _Nullable)datagram;

- (NSUInteger)hash;

@end

// TODO Documentation
@interface TTLServiceMutableDrivingProfileDatagram : TTLServiceDrivingProfileDatagram
/** Begin of the time frame */
@property(readwrite) NSDate * _Nullable begin;

/** End of the time frame */
@property(readwrite) NSDate * _Nullable end;

/** The time when the chunk arrived at the smart phone as unix timestamp. */
@property(readwrite) NSDate * _Nullable receivingTime;

// Scalar values

/** Mileage in meters */
@property(readwrite) NSNumber * _Nullable mileage;

/** Accumulated time with RPM > 0 in seconds */
@property(readwrite) NSNumber * _Nullable drivingTime;

/** Sum of idling periods in seconds */
@property(readwrite) NSNumber * _Nullable idleTime;

/** Maximum speed in km/h */
@property(readwrite) NSNumber * _Nullable maxSpeed;

// Histograms
/* Time driven in certain speed ranges */
@property(readwrite) TTLDrivingProfileHistogram * _Nullable drivingTimeBySpeed;

/*Distance traveled in certain speed ranges*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable mileageBySpeed;

/*Number of braking events within a g force range*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable braking;

/*Number of speedup events within a g force range*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable speedUp;

/*Number of cornering left events within a g force range*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable corneringLeft;

/*Number of cornering right events within a g force range*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable corneringRight;

/*Engine run time at different rpm ranges*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable engineRpm;

/*Engine run time at different load situations*/
@property(readwrite) TTLDrivingProfileHistogram * _Nullable engineLoad;

@end

/** Driving Profile Response
*
* @see [TTLServiceClient activateDrivingProfileRealTimeTransmission:]
* @see [TTLServiceDrivingProfileRealTimeDelegate onDrivingProfileDatagram:deviceId:]
* @see TTLServiceDrivingProfileDatagram
*/
@interface TTLServiceDrivingProfileRealTimeResponse : NSObject

/** The driving profile datagram.
* @see TTLServiceDrivingProfileDatagram
*/
@property TTLServiceDrivingProfileDatagram * _Nonnull drivingProfileDatagram;

- (NSString * _Nonnull)description;

@end

/** Delegate for receiving driving profile information.
*
* @see [TTLServiceClient activateDrivingProfileRealTimeTransmission:]
* @see TTLServiceDrivingProfileRealTimeResponse
* @see TTLServiceDrivingProfileDatagram
*/
@protocol TTLServiceDrivingProfileRealTimeDelegate <NSObject>

/** Called when a new driving profile chunk arrived.
*
*
* @param response TTLServiceDrivingProfileRealTimeResponse A response that contains the driving profile information.
*
* @see [TTLServiceClient activateDrivingProfileRealTimeTransmission:]
* @see TTLServiceDrivingProfileDatagram
*/
- (void)onDrivingProfileDatagram:(TTLServiceDrivingProfileRealTimeResponse * _Nonnull)response deviceId:(NSString * _Nonnull)deviceIdentifier;
@end
