import Foundation

/// Holds the details for connecting to the available backend systems.
class BackendReslover
{
    private let backendSystems : [BackendType:BackendConfig]
    
    private init() {
        let path = Bundle.main.path(forResource: "Backend", ofType: "plist")!
        let backendDict = NSDictionary(contentsOfFile: path)!
        backendSystems = [
            BackendType.Prod : BackendReslover.backendConfig(from: backendDict, forKey: "prod"),
            BackendType.Dev : BackendReslover.backendConfig(from: backendDict, forKey: "dev")
        ]
    }
    
    private static func backendConfig(from dict: NSDictionary, forKey key: String) -> BackendConfig {
        let backendDict = dict[key] as! NSDictionary
        return BackendConfig(
            connectDomain: backendDict["connectDomain"] as! String,
            connectApiKey: backendDict["connectApiKey"] as! String,
            gprsQueueDomain: backendDict["gprsQueueDomain"] as! String,
            gprsQueuePort: backendDict["gprsQueuePort"] as! Int
        )
    }
    
    func getBackendConfig(type: BackendType) -> BackendConfig {
        return backendSystems[type]!
    }
    
    static let sharedInstance = BackendReslover()
}
