/**
 Initalizes all central services used by the app.
 This includes The TTLServiceClient client in particular,
 which can be configured and initalized only once.
 */
class ServiceFactory {
    private static let sharedInstance = ServiceFactory()
    private let serviceClient: TTLServiceClient
    private let connectionService: ConnectionService
    private let userService = UserService()

    /// Access to the SDKs functionality.
    static var serviceClient: TTLServiceClient {
        get {
            return sharedInstance.serviceClient
        }
    }
    
    /// SDK related Connection information about devices and backend.
    static var connectionService: ConnectionService {
        get {
            return sharedInstance.connectionService
        }
    }
    
    /// Access to user related information
    static var userService: UserService {
        get {
            return sharedInstance.userService
        }
    }

    private init() {
        let conf = TTLServiceClientConfiguration()
        // Disable automatic activation to be in full control of the activation process.
        // Otherwise activation would run automatically as soon device and backend connections
        // are established.
        conf.automaticActivation = false;
        // Note that this call needs to be placed before the first call to TTLServiceClient.sharedInstance()
        TTLServiceClient.configureServiceClient(conf)
        serviceClient = TTLServiceClient.sharedInstance()
        connectionService = ConnectionService(serviceClient: serviceClient)
        
    }
}
