import Security

// Keychain Configuration
struct KeychainConfiguration {
    static let serviceName = "LINK100-SDK-Demo"
    static let accessGroup: String? = nil
}

/**
 Service that stores user related information of the Demo App,
 like the userName.
 */
class UserService: NSObject {
    private let userDefaults = UserDefaults.standard
    
    var hasCredentials: Bool {
        get {
            return userDefaults.string(forKey: "userName") != nil
        }
    }
    
    var credentials: Credentials {
        get {
            let keychainPasswordItem = KeychainPasswordItem(
                service: KeychainConfiguration.serviceName,
                account: userName,
                accessGroup: KeychainConfiguration.accessGroup
            )
            let keychainPassword = try! keychainPasswordItem.readPassword()
            return Credentials(account: userName, password: keychainPassword)
        }
        set (newCredentials) {
            let keychainPasswordItem = KeychainPasswordItem(
                service: KeychainConfiguration.serviceName,
                account: newCredentials.account,
                accessGroup: KeychainConfiguration.accessGroup
            )
            
            try! keychainPasswordItem.savePassword(newCredentials.password)
            userName = newCredentials.account
        }
    }
    
    func deleteCredentials() {
        guard let userName = userDefaults.string(forKey: "userName") else {return}
        
        let keychainPasswordItem = KeychainPasswordItem(
            service: KeychainConfiguration.serviceName,
            account: userName,
            accessGroup: KeychainConfiguration.accessGroup
        )
        
        try! keychainPasswordItem.deleteItem()
        userDefaults.removeObject(forKey: "userName")
    }
    
    var deviceIdentifier: String
    {
        get {
            return userDefaults.string(forKey: "deviceIdentifier")!
        }
        set(newDeviceIdentifier) {
            userDefaults.setValue(newDeviceIdentifier, forKey: "deviceIdentifier")
        }
    }
    
    var userName: String {
        get {
            return userDefaults.string(forKey: "userName")!
        }
        set(newUserName) {
            userDefaults.setValue(newUserName, forKey: "userName")
        }
    }
    
    var backend: BackendType {
        get {
            return BackendType(rawValue: userDefaults.string(forKey: "backend")!)!
        }
        set(newBackend) {
            userDefaults.setValue(newBackend.rawValue, forKey: "backend")
        }
    }
}
