import UIKit

protocol Validator {
    func validate(_ textField: UITextField) throws -> String
}

struct ValidatorError : Error {
    weak var source: UITextField?
}

class ValidatorFactory {
    static func validatorFor(type: ValidatorType) -> Validator {
        switch type {
        case .required: return RequiredFieldValidator()
        }
    }
}

class RequiredFieldValidator : Validator
{
    func validate(_ textField: UITextField) throws -> String {
        guard let value = textField.text?.trimmingCharacters(in: .whitespacesAndNewlines), !value.isEmpty else
        {
            throw ValidatorError(source: textField)
        }
        return value
    }
}
